
subroutine CalcEnergy(Pos, AtomIDs, excited, L, rcutbuck, rcutcoul, PEnergy, Dim, NAtom) !typical rc=12-15
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NAtom-1) :: AtomIDs
    real(8), intent(in) :: L, rcutbuck, rcutcoul
    logical, intent(in) :: excited
    real(8), intent(out) :: PEnergy
    real(8), parameter :: kalo = 1725.2, palo = 0.28971, calo = 0  ! Al parameters for buckingham potential
    real(8), parameter :: kyo = 1766.4, pyo = 0.33849, cyo = 19.43  ! Y parameters
    real(8), parameter :: koo = 9547.96, poo = 0.2192, coo = 32 ! O parameters
    real(8), parameter :: alpha = 0.2
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: alShift, yShift, ceShift, oShift, Shift, d2, dsqrt, id2, id6, pi, sqrtpi, rcutcoul2, rcutbuck2, invrcutbuck6
    real(8) :: kceo, pceo, cceo ! Ce buckingham parameters
    real(8) :: k, p, c ! buckingham parameters
    integer :: i, j, qi, qj
        if (excited .eqv. .False.) then
            kceo = 2010.18
            pceo = 0.348
            cceo = 23.11
        else if (excited .eqv. .True.) then
            kceo = 2010.18
            pceo = 0.3558
            cceo = 23.11
        endif
        pi=2*ACOS(0.0)
        sqrtpi=sqrt(pi)
        PEnergy=0
        rcutcoul2=rcutcoul*rcutcoul
        rcutbuck2=rcutbuck*rcutbuck
        invrcutbuck6=1/(rcutbuck2*rcutbuck2*rcutbuck2)
        
        alShift=-kalo*Exp(-rcutbuck/palo)+calo*invrcutbuck6  ! shifts for buckingham potential
        yShift=-kyo*Exp(-rcutbuck/pyo)+cyo*invrcutbuck6
        ceShift=-kceo*Exp(-rcutbuck/pceo)+cceo*invrcutbuck6
        oShift=-koo*Exp(-rcutbuck/poo)+coo*invrcutbuck6

        do i=0, NAtom-1
            Posi=Pos(i,:)
            if (AtomIDs(i)==4) then !set charge, -2 for O 3 for Al, Y, and Ce
                qi=-2
            else
                qi=3
            endif
            do j=i+1, NAtom-1
                if (AtomIDs(j)==4) then  !set charge, -2 for O 3 for Al, Y, and Ce
                    qj=-2
                else
                    qj=3
                endif
                rij=Pos(j,:)-Posi
                rij=rij-L*dnint(rij/L)
                d2 = sum(rij * rij)
                if (d2<rcutcoul2) then  !cutoff for long-range coulombic forces
                    dsqrt=sqrt(d2)
                    
                    !in joules
                    PEnergy=PEnergy + &
                    1.60217646*10.**(-19.)*1.60217646*10.**(-19.)*10.**(10.) &
                    /(4*pi*8.85418781762*10.**(-12.))*&
                    qi*qj*(ERFC(alpha*dsqrt)/dsqrt &
                     - ERFC(alpha*rcutcoul)/rcutcoul + (ERFC(alpha*rcutcoul)/(rcutcoul2) + &
                    2*alpha*EXP(-alpha*alpha*rcutcoul2)/(sqrtpi*rcutcoul))*(dsqrt-rcutcoul))

                    if (d2<rcutbuck2) then  !cutoff for short range repulsive interactions
                        id2 = 1. / d2       !inverse squared distance
                        id6 = id2 * id2 * id2   !inverse sixth distance
                        if (AtomIDs(i)==4) then  ! if atom i is O
                            if (AtomIDs(j)==1 .or. AtomIDs(j)==3) then !atom j is Al
                                k=kalo
                                p=palo
                                c=calo
                                Shift=alShift
                            else if (AtomIDs(j)==2) then ! atom j is Y
                                k=kyo
                                p=pyo
                                c=cyo
                                Shift=yShift
                            else if (AtomIDs(j)==5) then ! atom j is Ce
                                k=kceo
                                p=pceo
                                c=cceo
                                Shift=ceShift
                            endif
                            !in joules
                            PEnergy = PEnergy + (k*EXP(-dsqrt/p)-c*id6+Shift)*(1.602*10.**(-19.))
                        else if (AtomIDs(j)==4) then ! if atom j is O
                            if (AtomIDs(i)==1 .or. AtomIDs(i)==3) then ! Al
                                k=kalo
                                p=palo
                                c=calo
                                Shift=alShift
                            else if (AtomIDs(i)==2) then ! Y
                                k=kyo
                                p=pyo
                                c=cyo
                                Shift=yShift
                            else if (AtomIDs(i)==4) then ! O
                                k=koo
                                p=poo
                                c=coo
                                Shift=oShift
                            else if (AtomIDs(i)==5) then ! Ce
                                k=kceo
                                p=pceo
                                c=cceo
                                Shift=ceShift
                            endif
                            !in joules
                            PEnergy = PEnergy + (k*EXP(-dsqrt/p)-c*id6+Shift)*(1.602*10.**(-19.))
                        endif
                    endif
                endif
            enddo
        enddo
end subroutine


subroutine CalcEnergyForces(Pos, AtomIDs, excited, L, rcutbuck, rcutcoul, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    integer, intent(in), dimension(0:NAtom-1) :: AtomIDs
    real(8), intent(in) :: L, rcutbuck, rcutcoul
    logical, intent(in) :: excited
    real(8), intent(out) :: PEnergy
    real(8), parameter :: kalo = 1725.2, palo = 0.28971, calo = 0  ! Al parameters for buckingham potential
    real(8), parameter :: kyo = 1766.4, pyo = 0.33849, cyo = 19.43  ! Y parameters
    real(8), parameter :: koo = 9547.96, poo = 0.2192, coo = 32 ! O parameters
    real(8), parameter :: alpha = 0.2
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: alShift, yShift, ceShift, oShift, Shift, d2, dsqrt, id2, id6 
    real(8) :: pi, sqrtpi, rcutcoul2, rcutbuck2, invrcutbuck6
    real(8) :: kceo, pceo, cceo ! Ce buckingham parameters
    real(8) :: k, p, c ! buckingham parameters
    integer :: i, j, qi, qj

        if (excited .eqv. .False.) then
            kceo = 2010.18
            pceo = 0.348
            cceo = 23.11
        else if (excited .eqv. .True.) then
            kceo = 2010.18
            pceo = 0.3558
            cceo = 23.11
        endif
        pi=2*ACOS(0.0)
        sqrtpi=sqrt(pi)
        PEnergy=0
        Forces=0
        rcutcoul2=rcutcoul*rcutcoul
        rcutbuck2=rcutbuck*rcutbuck
        invrcutbuck6=1/(rcutbuck2*rcutbuck2*rcutbuck2)
        
        alShift=-kalo*Exp(-rcutbuck/palo)+calo*invrcutbuck6  ! shifts for buckingham potential
        yShift=-kyo*Exp(-rcutbuck/pyo)+cyo*invrcutbuck6
        ceShift=-kceo*Exp(-rcutbuck/pceo)+cceo*invrcutbuck6
        oShift=-koo*Exp(-rcutbuck/poo)+coo*invrcutbuck6

        do i=0, NAtom-1
            Posi=Pos(i,:)
            if (AtomIDs(i)==4) then !set charge, -2 for O 3 for Al, Y, and Ce
                qi=-2
            else
                qi=3
            endif
            do j=i+1, NAtom-1
                if (AtomIDs(j)==4) then  !set charge, -2 for O 3 for Al, Y, and Ce
                    qj=-2
                else
                    qj=3
                endif
                rij=Pos(j,:)-Posi
                rij=rij-L*dnint(rij/L)
                d2 = sum(rij * rij)
                if (d2<rcutcoul2) then  !cutoff for long-range coulombic forces
                    id2 = 1. / d2       !inverse squared distance
                    id6 = id2 * id2 * id2   !inverse sixth distance
                    dsqrt=sqrt(d2)

                    !in joules
                    PEnergy=PEnergy + &
                    1.60217646*10.**(-19.)*1.60217646*10.**(-19.)*10.**(10.)&
                    /(4*pi*8.85418781762*10.**(-12.))*&
                    qi*qj*(ERFC(alpha*dsqrt)/dsqrt - ERFC(alpha*rcutcoul)/rcutcoul + &
                    (ERFC(alpha*rcutcoul)/(rcutcoul2)+2*alpha*EXP(-alpha*alpha*rcutcoul2)/(sqrtpi*rcutcoul))*(dsqrt-rcutcoul))
                    !
                    Fij=-1.60217646*10.**(-19.)*1.60217646*10.**(-19.)*10.**(10.)*10.**(10.) &
                    /(4*pi*8.85418781762*10.**(-12.))* &
                    rij/dsqrt*qi*qj*(ERFC(alpha*dsqrt)*id2+2*alpha*Exp(-alpha*alpha*d2)/(sqrtpi*dsqrt) - & 
                    Erfc(alpha*rcutcoul)/(rcutcoul2)-2*alpha*Exp(-alpha*alpha*rcutcoul2)/(sqrtpi*rcutcoul))

                    Forces(i,:) = Forces(i,:) + Fij
                    Forces(j,:) = Forces(j,:) - Fij

                    if (d2<rcutbuck2) then  !cutoff for short range buckingham interactions
                        if (AtomIDs(i)==4) then  ! if atom i is O
                            if (AtomIDs(j)==1 .or. AtomIDs(j)==3) then !atom j is Al
                                k=kalo
                                p=palo
                                c=calo
                                Shift=alShift
                            else if (AtomIDs(j)==2) then ! atom j is Y
                                k=kyo
                                p=pyo
                                c=cyo
                                Shift=yShift
                            else if (AtomIDs(j)==5) then ! atom j is Ce
                                k=kceo
                                p=pceo
                                c=cceo
                                Shift=ceShift
                            endif
                            PEnergy = PEnergy + (k*EXP(-dsqrt/p)-c*id6+Shift)*(1.602*10.**(-19.))
                            Fij=-rij/dsqrt*(k/p*Exp(-dsqrt/p)-6*c*id6/dsqrt)*(1.602*10.**(-19.)*10.**10.)
                            Forces(i,:) = Forces(i,:) + Fij
                            Forces(j,:) = Forces(j,:) - Fij
                        else if (AtomIDs(j)==4) then ! if atom j is O
                            if (AtomIDs(i)==1 .or. AtomIDs(i)==3) then ! Al
                                k=kalo
                                p=palo
                                c=calo
                                Shift=alShift
                            else if (AtomIDs(i)==2) then ! Y
                                k=kyo
                                p=pyo
                                c=cyo
                                Shift=yShift
                            else if (AtomIDs(i)==4) then ! O
                                k=koo
                                p=poo
                                c=coo
                                Shift=oShift
                            else if (AtomIDs(i)==5) then ! Ce
                                k=kceo
                                p=pceo
                                c=cceo
                                Shift=ceShift
                            endif
                            PEnergy = PEnergy + (k*EXP(-dsqrt/p)-c*id6+Shift)*(1.602*10.**(-19.))
                            Fij=-rij/dsqrt*(k/p*Exp(-dsqrt/p)-6*c*id6/dsqrt)*(1.602*10.**(-19.)*10.**10.)
                            Forces(i,:) = Forces(i,:) + Fij
                            Forces(j,:) = Forces(j,:) - Fij
                        endif
                    endif
                endif
            enddo
        enddo
end subroutine

subroutine VVIntegrate(Pos, AtomIDs, Vel, Accel, AtomMasses, L, excited, rcutbuck, rcutcoul, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    logical, intent(in) :: excited
    real(8), intent(in) :: L, rcutbuck, rcutcoul, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    integer, intent(in), dimension(0:NAtom-1) :: AtomIDs
    real(8), intent(in), dimension(0:NAtom-1) :: AtomMasses
    real(8), intent(out) :: KEnergy, PEnergy
    real(8), dimension(0:NAtom-1, 0:Dim-1) :: Forces, AtomMasses3
    integer :: i
    external :: CalcEnergyForces
    do i=0, 2
        AtomMasses3(:,i)=AtomMasses
    enddo
    Forces=AtomMasses3*Accel
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, AtomIDs, excited, L, rcutbuck, rcutcoul, PEnergy, Forces, Dim, NAtom)
    Accel=Forces/AtomMasses3
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(AtomMasses3*Vel*Vel)
end subroutine